require "json"
require "fileutils"
require "asm"
require "asm/private_util"
require "rest_client"
require "logger"
require "asm/service"

module ASM
  class PortView
    attr_reader :logger, :server

    # @param server [Type::Server]
    # @param logger [Logger]
    def initialize(server, logger)
      @server = server
      @logger = logger
      @data_version = 1
    end

    # Retrieves portview data for a specific server
    #
    # @note used by {ASM.get_server_info}
    # @param deployment_id [String] deployment ID
    # @param puppet_certname [String] certname to retrieve
    # @return [Hash] cached port view data or empty hash
    def server_info(deployment_id, puppet_certname)
      logger.debug("Getting port view info for %s : %s" % [deployment_id, puppet_certname])

      deployment = deployment_data(deployment_id)

      return {} unless deployment

      @server = deployment_service(deployment, deployment_id).servers.find do |candidate|
        candidate.puppet_certname.downcase == puppet_certname.downcase
      end

      return {} unless server

      read_cache
    end

    # Creates a {Service} instance from a raw deployment
    #
    # @note used by {ASM.get_server_info}
    # @param deployment [Hash] deployment from a JSON file
    # @param deployment_id [String] the id for the deployment
    # @return [Service]
    def deployment_service(deployment, deployment_id)
      Service.new(deployment, :deployment => ServiceDeployment.new(deployment_id, Data::Deployment.new(ASM.database)))
    end

    # Retrieves the network overview data for a specific server
    #
    # @note this only returns a subset of keys, those used by AsmManager
    # @see Provider::Server#network_overview
    # @return [Hash,nil] nil when server was not passed into the constructor
    def network_overview
      return nil unless server # a bit sloppy but the old code did the same

      manager_keys = [
        :connected_switches,
        :fc_interfaces,
        :fcoe_interfaces,
        :name,
        :network_config,
        :physical_type,
        :razor_policy_name,
        :related_switches,
        :serial_number,
        :server
      ]

      server.network_overview.select {|k, _v| manager_keys.include?(k) }
    end

    # Determines the cache file name for a certname
    #
    def cache_filename
      "/opt/Dell/ASM/cache/%s_portview.json" % server.puppet_certname.downcase
    end

    # Writes the cache file for a server
    #
    # @return [Hash] the written portview data
    # @raise [StandardError] for IO errors etc
    def write_cache
      logger.debug("Writing the cache for %s at %s with data version %s" % [server.puppet_certname, cache_filename, @data_version])

      portview_data = updated_port_view_data

      File.write(cache_filename, portview_data.to_json)

      portview_data
    end

    # Determines if a valid cache exist for a cert name
    #
    # @return [Boolean]
    def has_cache?
      return false unless File.exist?(cache_filename)
      return false unless File.file?(cache_filename)
      return false if File.symlink?(cache_filename)
      return false if File.zero?(cache_filename)
      return false unless File.readable?(cache_filename)

      true
    end

    # Deletes the network topology cache for a specific server
    def delete_cache
      if File.exist?(cache_filename)
        logger.info("Deleting server network topolocy cache in %s" % cache_filename)
        File.unlink(cache_filename)
      else
        logger.info("No server network topology cache found in %s" % cache_filename)
      end
    end

    # Reads the previously stored powerview cache for a server
    #
    # @return [Hash]
    def read_cache
      force_update = false

      if has_cache?
        logger.debug("Reading portview cache for server %s from %s" % [server.puppet_certname, cache_filename])

        portview_data = JSON.parse(File.read(cache_filename))

        if portview_data["DATA_VERSION"] != @data_version
          logger.debug("Cache exists for %s, but the data version is %s while we want %s, will update cache" %
                       [server.puppet_certname, portview_data["DATA_VERSION"], @data_version])

          force_update = true
        end
      else
        logger.debug("No cache exists for %s, will create a fresh cache" % server.puppet_certname)
        force_update = true
      end

      portview_data = write_cache if force_update
      portview_data.delete("DATA_VERSION")
      portview_data
    end

    # Extracts only expected keys from port view data
    #
    # @see #network_overview
    # @return [Hash] updated info
    def updated_port_view_data
      port_view_info = network_overview.clone

      interface_keys = ["id", "name", "redundancy", "enabled", "usedforfc", "partitioned", "nictype", "interfaces", "fabrictype", "card_index"]
      inner_interface_keys = ["id", "name", "partitioned", "partitions", "enabled", "redundancy", "nictype", "fqdd"]
      partition_keys = [
        "id", "name", "networks", "networkObjects", "minimum", "maximum", "lanMacAddress",
        "iscsiMacAddress", "iscsiIQN", "wwnn", "wwpn", "port_no", "partition_no", "partition_index", "fqdd", "mac_address"
      ]

      network_config = port_view_info[:network_config].select {|k, _v| ["id", "interfaces"].include?(k) }

      all_outer_interfaces = []
      network_config["interfaces"].each do |each_outer_interface|
        all_inner_interfaces = []

        this_outer_interface = each_outer_interface.select {|k, _v| interface_keys.include?(k) }
        all_outer_interfaces << this_outer_interface

        this_outer_interface["interfaces"].each do |each_inner_interface|
          all_partitions = []

          inner_interfaces_for_this_outer_interface = each_inner_interface.select {|k, _v| inner_interface_keys.include?(k) }
          all_inner_interfaces << inner_interfaces_for_this_outer_interface

          inner_interfaces_for_this_outer_interface["partitions"].each do |each_partition_in_this_interface|
            all_partitions << each_partition_in_this_interface.select {|k, _v| partition_keys.include?(k) }
          end

          inner_interfaces_for_this_outer_interface["partitions"] = all_partitions
        end

        this_outer_interface["interfaces"] = all_inner_interfaces
      end

      network_config["interfaces"] = all_outer_interfaces
      port_view_info[:network_config] = network_config
      port_view_info["DATA_VERSION"] = @data_version

      port_view_info
    end

    # Determines if a file is absent or zero bytes
    #
    # @param filename [String, nil]
    # @return [Boolean]
    def file_absent_or_zero?(filename)
      return true unless filename

      !File.exist?(filename) || File.zero?(filename)
    end

    # Returns a list of all JSON file for a deployment
    #
    # Certain deployment retries will write multiple deployment.json
    # files in the main deployment dir so this returns the list sorted
    # by ctime, assuming the clocks are reliable.
    #
    # @param deployment_id [String] deployment id
    # @return [Array<String>] sorted by ctime
    def deployment_json_files(deployment_id)
      json_files = File.join(ASM.base_dir, deployment_id, "deployment*.json")
      Dir.glob(json_files).sort_by {|v| File.ctime(v)}
    end

    # Reads the deployment.json for a specified deployment
    #
    # If the deployment.json does not exist retries will be done until
    # it is found for up to 10 seconds, this allows for the period between
    # the POST of the data and the service starting
    #
    # @note used by {ASM.get_server_info}
    # @param deployment_id [String] deployment id
    # @return [Hash,nil] deployment data
    def deployment_data(deployment_id)
      10.times do |try|
        file_name = deployment_json_files(deployment_id).last

        if file_absent_or_zero?(file_name)
          logger.debug("%s does not exist on try %d/10" % [file_name, try + 1])

          sleep 1 unless try == 9
        else
          logger.debug("%s found for deployment %s after %d tries" % [file_name, deployment_id, try])
          return JSON.parse(File.read(file_name), :max_nesting => 100)
        end
      end

      nil
    end
  end
end
