require "asm/private_util"
require "asm/network_configuration"
require "asm/processor/linux_post_os"

module ASM
  module Processor
    class LinuxVMPostOS < ASM::Processor::LinuxPostOS
      def logger
        @resource.deployment.logger
      end

      def default_gateway_network_config
        logger.debug("Looking for network by the id, %s" % @resource.default_gateway)
        def_network = @resource.network_interfaces.find {|network| network["id"] == @resource.default_gateway}
        if def_network
          if supported_suse? && !@resource.clone?
            {"network::suse_routes" => {
              "gateway" => def_network["staticNetworkConfiguration"]["gateway"] || "",
              "netmask" => def_network["staticNetworkConfiguration"]["subnet"]
            }}
          else
            {"network::global" => {"gateway" => def_network["staticNetworkConfiguration"]["gateway"] || ""}}
          end
        else
          {}
        end
      end

      def post_os_config(puppet_hash={})
        logger.debug("Setting the default gateway if requested")
        puppet_hash["classes"] ||= {}
        puppet_hash["classes"].merge!(default_gateway_network_config)
        logger.debug("Starting to process puppet resources for VM static NIC")
        process_network_config(puppet_hash)
        unless puppet_hash["resources"].empty? && puppet_hash["classes"].empty?
          appliance_ip = host_ip_config(@resource.network_interfaces)
          puppet_hash["resources"]["host"] = {"dellasm" => {"ensure" => "present", "ip" => appliance_ip}}
        end
        logger.debug("Linux Post OS puppet config: #{puppet_hash}")

        puppet_hash
      end

      def process_network_config(puppet_hash={})
        net_config = {}
        seq = 0

        # @vm resource object should have had process! called on it.
        # If vm hasn't been fully deployed yet, we still need 1 extra interface
        # to account for the PXE network. We need to have both interfaces up and
        # running at the same time so we can switch host file entry without
        # breaking the vm's networking
        unless @resource.deployed?
          if supported_suse? && !@resource.clone?
            net_config["network::if::suse_dynamic"] ||= {}
            net_config["network::if::suse_dynamic"][seq] = {
              "ensure" => "up"
            }
          end
          seq += 1
        end
        @resource.network_interfaces.each do |network|
          logger.debug("Processing network, %s, into puppet config" % network.inspect)
          if supported_suse? && !@resource.clone?
            process_suse_network_config(net_config, network, puppet_hash, seq)
          else
            process_rhel_network_config(net_config, network)
          end

          # the seq still has to be incremented to preserve the order of network interfaces on the VM
          seq += 1
        end
        puppet_hash["resources"] = net_config

        puppet_hash
      end

      def process_rhel_network_config(net_config, network)
        hash_network = Hashie::Mash.new(network)
        logger.debug("VM Network workload network: %s" % hash_network.inspect)
        vm_mac_address = @resource.macaddress(hash_network)

        if network["static"]
          net_config["network::if::redhat_static"] ||= {}
          net_config["network::if::redhat_static"][vm_mac_address] = {
            "ensure" => "up",
            "ipaddress" => network["staticNetworkConfiguration"]["ipAddress"],
            "netmask" => network["staticNetworkConfiguration"]["subnet"],
            "gateway" => (network["staticNetworkConfiguration"]["gateway"] || ""),
            "domain" => (network["staticNetworkConfiguration"]["dnsSuffix"] || ""),
            "defroute" => "no"
          }
          if network["id"] == @resource.default_gateway
            net_config["network::if::redhat_static"][vm_mac_address]["defroute"] = "yes"
            resolv_res = resolv(network)
            unless resolv_res.empty?
              resolv_res["ensure"] = "present"
              net_config["network::resolv"] = {"resolv_conf" => resolv_res}
            end
          end
        else
          net_config["network::if::redhat_dynamic"] ||= {}
          net_config["network::if::redhat_dynamic"][vm_mac_address] = {
            "ensure" => "up"
          }
        end
      end

      def process_suse_network_config(net_config, network, puppet_hash, seq)
        if network["static"]
          net_config["network::if::suse_static"] ||= {}
          net_config["network::if::suse_static"][seq] = {
            "ensure" => "up",
            "ipaddress" => network["staticNetworkConfiguration"]["ipAddress"],
            "netmask" => network["staticNetworkConfiguration"]["subnet"],
            "gateway" => (network["staticNetworkConfiguration"]["gateway"] || ""),
            "domain" => (network["staticNetworkConfiguration"]["dnsSuffix"] || "")
          }
          if network["id"] == @resource.default_gateway && puppet_hash["classes"]["network::suse_routes"]
            puppet_hash["classes"]["network::suse_routes"]["device"] = seq
            resolv_res = resolv(network)
            unless resolv_res.empty?
              resolv_res["ensure"] = "present"
              net_config["network::resolv"] = {"resolv_conf" => resolv_res}
            end
          end
        else
          net_config["network::if::suse_dynamic"] ||= {}
          net_config["network::if::suse_dynamic"][seq] = {
            "ensure" => "up"
          }
        end
      end
    end
  end
end
# linux_vm_post_os.rb
