require "asm/private_util"
require "asm/network_configuration"

module ASM
  module Processor
    class PostOS
      # Initialize
      #
      # @param [ASM::Type::[Server/Virtualmachine]] resource
      def initialize(resource)
        @resource = resource
        @required_resource = nil
      end

      def logger
        @resource.deployment.logger
      end

      def network_params
        @resource.component_configuration.fetch("asm::esxiscsiconfig", {})[@resource.puppet_certname]
      end

      def network_config
        return nil unless network_params
        @nc_config ||= begin
          network_config = ASM::NetworkConfiguration.new(network_params["network_configuration"], logger)

          network_config.add_nics!(@resource.device_config)

          network_config
        end
      end

      def asm_server
        @asm_server ||= begin
          @resource.component_configuration["asm::server"].fetch(@resource.uuid, {})
        end
      end

      def default_gateway_network
        if @resource.type == "VIRTUALMACHINE"
          @resource.default_gateway
        else
          network_params ? network_params.fetch("default_gateway", "") : ""
        end
      end

      def mtu
        @mtu ||= begin
          network_params ? network_params.fetch("mtu", "9000").to_i : 9000
        end
      end

      def applications
        @resource.service_component.associated_components("SERVICE")
      end

      # Returns hash of puppet module post-install resources/classes
      #
      # @example Returned data example:
      #   {"classes"=>{"linux_postinstall"=>{"install_packages"=>"httpd", "upload_recursive"=>"false"}, "haproxy"=>{"require"=>"Class['linux_postinstall']"}},
      #   "resources"=> {"haproxy::listen"=>{"puppet00"=>{"collect_exported"=>"false", "ipaddress"=>"0.0.0.0", "ports"=>"8080"}},
      #   "haproxy::balancermember"=>
      #   {"master00"=>{"listening_service"=>"puppet00", "require"=>"Haproxy::listen['puppet00']", "server_names"=>"master00.example.com", "ipaddresses"=>"10.0.0.10"}}}}
      # @return [Hash]
      def post_os_services
        services = {}
        classes = post_os_components("class")
        services["classes"] = classes unless classes.empty?

        resources = post_os_components("type")
        services["resources"] = resources unless resources.empty?
        services = create_mssql_password_tokens(services) if services["classes"] && services["classes"]["mssql2012"]
        services["classes"]["mssql2012"].delete("decrypt") if services["classes"] && services["classes"]["mssql2012"] && services["classes"]["mssql2012"]["decrypt"]

        if default_interval_config && !default_interval_config.empty?
          services["resources"] ||= {}
          services["resources"].merge!(default_interval_config)
        end

        services
      end

      # Returns puppet post-install components of a given type
      #
      # @param type [String] puppet install type (type | class)
      # @return [Hash]
      def post_os_components(type)
        result = {}
        components = applications.select { |c| c["service_type"] == type }.sort_by { |c| c["install_order"]}

        components.each do |component|
          component_hash = post_os_component(component["component"], type)
          if result[component_hash.keys.first]
            result[component_hash.keys.first].merge!(component_hash.values.first)
          else
            result.merge!(post_os_component(component["component"], type))
          end
          @required_resource = build_require_name(component["component"], type) unless type == "class"
        end
        result
      end

      def host_ip_config(networks)
        appliance_ip = ASM::Util.default_routed_ip
        networks.each do |network|
          if network["static"]
            preferred_ip = ASM::Util.get_preferred_ip(network["staticNetworkConfiguration"]["ipAddress"])
            return preferred_ip unless preferred_ip == appliance_ip
          end
        end
        appliance_ip
      end

      # Returns the hash with tokenized passwords
      #
      # @param component_config [Hash]
      # @return [Hash]
      def create_mssql_password_tokens(component_config)
        component_config["classes"]["mssql2012"].each do |k, _v|
          if k =~ /pwd|password/
            component_config["classes"]["mssql2012"][k] = ASM::PrivateUtil.create_password_token(component_config["classes"]["mssql2012"][k], @resource.agent_certname)
          end
        end

        component_config
      end

      # @note Placeholder method to be implemented by derived classes
      def default_interval_config
        {}
      end

      # Returns puppet hash for given post-install component
      #
      # @param component [ASM::Service::Component]
      # @return [Hash]
      def post_os_component(component, component_type)
        puppet_class = {}
        component_name = component.configuration.keys.first
        component_data = component.configuration.values.first
        component_title = component_data.keys.first
        component_config = ASM::PrivateUtil.build_component_configuration(component.to_hash, :decrypt => @resource.deployment.decrypt?)
        params = component_data.values.first

        if params.empty?
          puppet_class[component_name] = {}
          # @note: This is broken in our current puppet version
          # puppet_class[component_name]["require"] = require unless require.nil?
        end

        if component_type == "class"
          puppet_class[component_name] = component_config[component_name].values.first
          puppet_class[component_name].delete_if { |_, v| v.is_a?(String) && v.empty? }
        else
          puppet_class = component_config
          puppet_class[component_name][component_title].delete_if { |_, v| v.is_a?(String) && v.empty? }
          puppet_class[component_name][component_title]["require"] = @required_resource unless @required_resource.nil?
        end

        puppet_class
      end

      def build_require_name(component, type)
        name = component.configuration.keys.first
        if type == "class"
          require = "Class[%s]" % name.downcase
        else
          data = component.configuration.values.first
          title = data.keys.first
          require = "%s[%s]" % [name.split("::").each(&:capitalize!).join("::"), title]
        end
        require
      end
    end
  end
end
