require "asm/private_util"
require "asm/network_configuration"

module ASM
  module Processor
    module Server
      # Produce a asm::server, asm::idrac and hyperv::config from a resource hash
      #
      # @param title [String] puppet resource named for asm::idrac etc
      # @param old_resources [Hash] hash where newly created resources are appended to
      # @param target_ip [String] iSCSI target IP Address
      # @param vol_names [Array<String>] iSCSI volume names
      # @param logger [Logger] logging class
      # @param disk_part_flag [Boolean] hyperv::config hyperv_diskpart property
      # @param storage_type ["iscsi", "fc"]
      # @param storage_model ["equallogic", "compellent", "vnx"]
      # @param iscsi_fabric [String] fabric to connect to iSCSI
      # @param iscsi_macs [Array<String>] mac address for iscsi
      # @return [Hash]
      # @raise [StandardError] on invalid combination of options
      def self.munge_hyperv_server(title, old_resources, target_ip,
                                   vol_names, logger, disk_part_flag,
                                   storage_type="iscsi", storage_model="equallogic",
                                   iscsi_fabric="Fabric A",
                                   iscsi_macs=[])

        resources = old_resources.dup
        idrac_params = (resources["asm::idrac"] || {})[title]

        if idrac_params
          if idrac_params["target_boot_device"] == "SD"
            raise(ArgumentError, "HyperV does not work with target boot device SD")
          end
          idrac_params["enable_npar"] = false
        end

        server_params = ((resources["asm::server"] || {})[title] || {}).dup
        puppet_classification_data = {"hyperv::config" => {}}
        hv_config = puppet_classification_data["hyperv::config"]
        installer_options = server_params["installer_options"] || {}

        ["domain_name", "fqdn", "domain_admin_user", "domain_admin_password", "ntp_server"].each do |param|
          if param == "ntp_server"
            hv_config["ntp"] = installer_options.delete("ntp_server")
          elsif param == "domain_admin_password"
            hv_config["domain_admin_password"] = ASM::PrivateUtil.create_password_token(
              server_params["installer_options"]["domain_admin_password"],
              ASM::Util.hostname_to_certname(server_params["os_host_name"])
            )

            installer_options.delete(param)
          else
            hv_config[param] = installer_options.delete(param)
          end
        end

        server_params.delete("installer_options") if installer_options.empty?

        hv_config["iscsi_target_ip_address"] = target_ip
        hv_config["iscsi_volumes"] = vol_names

        net_params = (resources["asm::esxiscsiconfig"] || {})[title]

        network_config = ASM::NetworkConfiguration.new(net_params["network_configuration"], logger)
        device_conf = ASM::DeviceManagement.parse_device_config(title)

        unless device_conf["host"] == "127.0.0.1"
          network_config.add_nics!(device_conf, :add_partitions => true)
          mac_address = network_config.get_partitions("HYPERVISOR_MANAGEMENT").map(&:mac_address).compact.join(",")
        end

        hv_config["nic_team_member_macs"] = mac_address
        management_network = network_config.get_network("HYPERVISOR_MANAGEMENT")
        migration_network = network_config.get_network("HYPERVISOR_MIGRATION")
        private_network = network_config.get_network("HYPERVISOR_CLUSTER_PRIVATE")

        net_mapper = {
          "ipAddress" => "ip_address",
          "subnet"     => "netmask",
          "gateway"    => "gateway"
        }

        [management_network, migration_network, private_network].each do |network|
          param_prefix = "converged_net" if network["type"] == "HYPERVISOR_MANAGEMENT"
          param_prefix = "live_migration" if network["type"] == "HYPERVISOR_MIGRATION"
          param_prefix = "private_cluster" if network["type"] == "HYPERVISOR_CLUSTER_PRIVATE"

          hv_config["#{param_prefix}_vlan_id"] = network["vlanId"]

          net_mapper.each do |attr, puppet_param|
            param = "#{param_prefix}_#{puppet_param}"
            hv_config[param] = network["staticNetworkConfiguration"][attr]
            hv_config[param] = "" if network["staticNetworkConfiguration"][attr].nil?
          end

          if network["type"] == "HYPERVISOR_MANAGEMENT"
            hv_config["converged_net_dns_server"] = network["staticNetworkConfiguration"]["primaryDns"]
          end
        end

        storage_networks = network_config.get_networks("STORAGE_ISCSI_SAN")

        if storage_type == "iscsi"
          first_net = storage_networks[0]
          second_net = storage_networks[1]
          hv_config["iscsi_netmask"] = first_net["staticNetworkConfiguration"]["subnet"]
          hv_config["iscsi_vlan_id"] = first_net["vlanId"]
          hv_config["iscsi_fabric"] = iscsi_fabric
          hv_config["iscsi_macs"] = iscsi_macs.compact.join(",")
          hv_config["iscsi_ip_addresses"] = [
            first_net["staticNetworkConfiguration"]["ipAddress"],
            second_net["staticNetworkConfiguration"]["ipAddress"]
          ]
        end

        iscsi_networks = []

        network_config.get_all_partitions.each do |storage_partition|
          storage_partition["networkObjects"].each do |network_object|
            next unless network_object["type"] == "STORAGE_ISCSI_SAN"

            iscsi_networks.push([
              storage_partition["mac_address"],
              network_object["staticNetworkConfiguration"]["subnet"],
              network_object["staticNetworkConfiguration"]["ipAddress"],
              network_object["vlanId"]
            ].join(","))
          end
        end

        hv_config["iscsi_networks"] = iscsi_networks.join(";")
        hv_config["hyperv_diskpart"] = disk_part_flag

        if storage_type == "fc" && storage_model == "compellent"
          hv_config["pod_type"] = "AS1000"
        elsif storage_type == "fc" && storage_model == "vnx"
          hv_config["pod_type"] = "EMCFC"
        elsif storage_type == "iscsi" && storage_model == "compellent"
          hv_config["pod_type"] = "AS1000iSCSI"
        end

        # Get the appliance IP Address accessible from hypervisor managemen network
        hv_config["appliance_hypervior_management_ip"] = Util.get_preferred_ip(
          hv_config["converged_net_ip_address"]
        )

        server_params["puppet_classification_data"] = puppet_classification_data
        server_params["os_image_type"] = server_params["os_image_type"] || "windows"

        (resources["asm::server"] || {})[title] = server_params
        (resources["asm::idrac"] || {})[title] = idrac_params

        resources
      end
    end
  end
end
