require "asm/firmware"

module ASM
  class Provider
    class Configuration
      class Serverupdate < Provider::Base
        puppet_type "asm::server_update"

        property :install_type,           :default => "uri",          :validation => String
        property :force_restart,          :default => false,          :validation => :boolean
        property :path,                   :default => nil,            :validation => String
        property :asm_hostname,           :default => nil,            :validation => String
        property :esx_hostname,           :default => nil,            :validation => String
        property :esx_password,           :default => nil,            :validation => String
        property :server_firmware,        :default => nil,            :validation => String
        property :server_software,        :default => nil,            :validation => String
        property :instance_id,            :default => nil,            :validation => String
        property :vcenter_cert,           :default => nil,            :validation => String
        property :vcenter_ha_config,      :default => nil,            :validation => String
        property :vcenter_server_list,    :default => nil,            :validation => Array, :tag => :extra

        def server_firmware_update_munger(o_value, n_value)
          return nil if n_value == "[]" || n_value == "" # Force nil value if empty JSON array is passed from the controller
          return n_value if n_value.is_a?(String) || n_value.nil?
          JSON.dump(n_value)
        end

        def server_software_update_munger(o_value, n_value)
          return nil if n_value == "[]" || n_value == "" # Force nil value if empty JSON array is passed from the controller
          return n_value if n_value.is_a?(String) || n_value.nil?
          JSON.dump(n_value)
        end

        def vcenter_ha_config_update_munger(o_value, n_value)
          return n_value if n_value.is_a?(String)
          JSON.dump(n_value)
        end

        def vcenter_server_list_update_munger(o_value, n_value)
          return n_value if n_value.is_a?(Array)
          JSON.load(n_value)
        end

        def vcenter_cluster_maintenance_mode_check
          # Check the maintenance mode status for each of the server
          # Raise Error if any of the server is already in the maintenance mode

          return true unless vcenter_server_list

          resource_hash = {"esx_maintmode" => {}}
          vcenter_server_list.each do |cluster_host|
            resource_hash["esx_maintmode"].merge!(esx_maintenance_mode(cluster_host))
          end
          resource_hash.merge!(transport_config)
          type.process_generic(vcenter_cert, resource_hash, puppet_run_type, true)
        end

        def esx_maintenance_mode(server_name)
          {
            server_name => {
              "ensure" => "absent",
              "evacuate_powered_off_vms" => true,
              "timeout" => 0,
              "fail_when_undetermined" => false,
              "transport" => "Transport[vcenter]"
            }
          }
        end

        def transport_config
          {
            "transport" => {
              "vcenter" => {
                "name" => vcenter_cert,
                "options" => {"insecure" => true},
                "provider" => "device_file"
              }
            }
          }
        end

        def configure_hook
          unless asm_hostname
            self.asm_hostname = ASM::Util.get_preferred_ip(type.device_config.host) if type.device_config
          end
        end

        # Determines if this resource should be processed using puppet
        #
        # @return [Boolean]
        def requires_puppet?
          !!(esx_hostname || vcenter_cert)
        end

        # Process the firmware update job through ASM::Util instead of puppet run to process
        #
        # @return [void]
        def process!(options={})
          if requires_puppet?
            logger.debug("Ensure all servers are in not in maintenance mode")
            vcenter_cluster_maintenance_mode_check
            logger.debug("Processing %s using Puppet" % type.puppet_certname)
            super
          else
            ASM::Firmware.idrac_fw_install_from_uri(to_puppet, create_idrac_fw_nfs_path, type.device_config, logger)
            type.update_inventory if options[:update_inventory]
          end
        end

        # Generate nfs and mount path using firmware resource
        #
        # @return [Array<Hash>] The results of each instances with updated uri and mount path
        def create_idrac_fw_nfs_path
          firmwares = JSON.parse(server_firmware)

          firmwares.each do |firmware|
            full_path = firmware["uri_path"]
            full_path_split = full_path.split("/")
            mount = full_path_split[0..4].join("/")
            rel_path = full_path_split[5..-1].join("/")
            uri = "nfs://%s/%s;mountpoint=%s" % [asm_hostname, rel_path, mount]
            firmware["uri_path"] = uri
          end
          firmwares
        end
      end
    end
  end
end
