require "spec_helper"
require "asm/device_management"
require "asm"
require "tempfile"

describe ASM::DeviceManagement do
  before do
    ASM.init_for_tests
    @test_dir = Dir.mktmpdir("device_mgmt_test")
    @conf_dir = FileUtils.mkdir("#{@test_dir}/devices").first
    @node_data_dir = FileUtils.mkdir("#{@test_dir}/node_data").first
    @ssl_dir = FileUtils.mkdir("#{@test_dir}/ssl").first
    @cert_name = "foo-127.0.0.1"
    @tmpfile = Tempfile.new("AsmUtil_spec")

    ASM::DeviceManagement.send(:remove_const, :DEVICE_CONF_DIR)
    ASM::DeviceManagement.const_set(:DEVICE_CONF_DIR, @conf_dir)
    ASM::DeviceManagement.send(:remove_const, :NODE_DATA_DIR)
    ASM::DeviceManagement.const_set(:NODE_DATA_DIR, @node_data_dir)
    ASM::Util.send(:remove_const, :DEVICE_SSL_DIR)
    ASM::Util.const_set(:DEVICE_SSL_DIR, @ssl_dir)

    mock_log = mock("device_management")
    mock_log.stub_everything
    ASM::DeviceManagement.stubs(:logger).returns(stub(:debug => nil, :warn => nil, :info => nil))
  end

  after do
    ASM.reset
    FileUtils.remove_entry_secure @test_dir
    @tmpfile.unlink
  end

  it "should be able to parse single device config file" do
    certname = "equallogic-172.17.15.10"
    text = <<END
[#{certname}]
  type equallogic
  url https://eqluser:eqlpw@172.17.15.10

END
    @tmpfile.write(text)
    @tmpfile.close

    conf = ASM::DeviceManagement.parse_device_config_file(@tmpfile)
    expect(conf.keys.size).to eq 1
    expect(conf[certname].provider).to eq "equallogic"
  end

  it "should be able to parse uri query string" do
    certname = "equallogic-172.17.15.10"
    text = <<END
[#{certname}]
  type equallogic
  url https://eqluser:eqlpw@172.17.15.10?foo=bar

END
    @tmpfile.write(text)
    @tmpfile.close

    conf = ASM::DeviceManagement.parse_device_config_file(@tmpfile)
    ASM::DeviceManagement.stubs(:parse_device_config_file).returns(conf)
    File.stubs(:exist?).returns(true)

    conf = ASM::DeviceManagement.parse_device_config(certname)
    expect(conf.keys.size).to eq 12
    expect(conf.cert_name).to eq certname
    expect(conf.host).to eq "172.17.15.10"
    expect(conf.arguments).to eq("foo" => "bar")
  end

  it "should handle missing uri query string" do
    certname = "equallogic-172.17.15.10"
    text = <<END
[#{certname}]
  type equallogic
  url https://eqluser:eqlpw@172.17.15.10

END
    @tmpfile.write(text)
    @tmpfile.close

    conf = ASM::DeviceManagement.parse_device_config_file(@tmpfile)
    ASM::DeviceManagement.stubs(:parse_device_config_file).returns(conf)
    File.stubs(:exist?).returns(true)

    conf = ASM::DeviceManagement.parse_device_config(certname)
    expect(conf.keys.size).to eq 12
    expect(conf.cert_name).to eq certname
    expect(conf.host).to eq "172.17.15.10"
    expect(conf.arguments).to eq({})
  end

  it "should be able to delete a node data file" do
    conf_file = FileUtils.touch("#{@node_data_dir}/#{@cert_name}.yaml").first
    ASM::DeviceManagement.remove_node_data(@cert_name)
    expect(File.exist?(conf_file)).to eq(false)
  end

  it "should not fail if node data file does not exist" do
    ASM::DeviceManagement.remove_node_data(@cert_name)
  end

  it "should be able to delete a conf file" do
    conf_file = FileUtils.touch("#{@conf_dir}/#{@cert_name}.conf").first
    ASM::DeviceManagement.remove_device_conf(@cert_name)
    expect(File.exist?(conf_file)).to eq(false)
  end

  it "should be able to delete a device puppet ssl folder" do
    dir_name = @ssl_dir + "/#{@cert_name}"
    dir = FileUtils.mkdir_p(dir_name)
    # Sanity check to ensure proper directory was created
    # ASM::Util.run_command_simple("sudo /opt/Dell/scripts/rm-device-ssl.sh #{device_name}")
    ASM::Util.stubs(:run_command_simple).with("sudo /opt/Dell/scripts/rm-device-ssl.sh #{@cert_name}") do
      FileUtils.rm_rf(dir)
    end.returns(true)
    expect(File.exist?(dir_name)).to eq(true)
    ASM::DeviceManagement.remove_device_ssl_dir(@cert_name)
    expect(File.exist?(dir_name)).to eq(false)
  end

  it "should be able to clean up devices if puppet cert is active" do
    node_data_file = FileUtils.touch(ASM::DeviceManagement::NODE_DATA_DIR + "/#{@cert_name}.yaml").first
    conf_file = FileUtils.touch(ASM::DeviceManagement::DEVICE_CONF_DIR + "/#{@cert_name}.conf").first

    # NOTE: not actually creating this directory, just verifying rm-device-ssl.sh is run
    ssl_dir = ASM::Util::DEVICE_SSL_DIR + "/#{@cert_name}"

    ASM::PrivateUtil.stubs(:get_puppet_certs).returns([@cert_name])

    ASM::Util.stubs(:run_command_simple)
             .with("sudo /opt/Dell/scripts/rm-device-ssl.sh #{@cert_name}")
             .returns(Hashie::Mash.new(:exit_status => 0))
    ASM::Util.expects(:run_command_simple)
             .with("sudo puppet cert clean #{@cert_name}")
             .returns(Hashie::Mash.new(:exit_status => 0))
    ASM::Util.expects(:run_command_simple)
             .with("sudo puppet node deactivate --terminus=puppetdb #{@cert_name}")
             .returns(Hashie::Mash.new(:exit_status => 0))

    ASM::DeviceManagement.remove_device(@cert_name)
    expect(File.exist?(node_data_file)).to eq(false)
    expect(File.exist?(conf_file)).to eq(false)
    expect(File.exist?(ssl_dir)).to eq(false)
  end

  it "should report unknown devices as such" do
    expect(ASM::DeviceManagement.get_device_state("unknown_device")).to eq(:unknown)
  end

  it "should check state validity" do
    expect do
      ASM::DeviceManagement.set_device_state("rspec", :rspecstate)
    end.to raise_error("Unsupported state rspecstate for node rspec")
  end

  it "should be able to set and retrieve the device state" do
    ASM::DeviceManagement.set_device_state("rspec", :failed)
    expect(ASM::DeviceManagement.get_device_state("rspec")).to eq(:failed)
  end

  describe "when writing a config file" do
    before do
      @device = {"cert_name" => "rspec", "host" => "1.2.3.4",
                 "user" => "rip", "pass" => "secret",
                 "provider" => "rspec", "scheme" => "ssh"}
    end

    it "should correctly create URLs" do
      expect(ASM::DeviceManagement.url_for_device(@device)).to eq("ssh://rip:secret@1.2.3.4")
      expect(ASM::DeviceManagement.url_for_device(@device.merge("user" => nil, "pass" => nil))).to eq("ssh://1.2.3.4")
      expect(ASM::DeviceManagement.url_for_device(@device.merge("port" => 443))).to eq("ssh://rip:secret@1.2.3.4:443")
      expect(ASM::DeviceManagement.url_for_device(@device.merge("path" => "rspec/rspec"))).to eq("ssh://rip:secret@1.2.3.4/rspec/rspec")
      expect(ASM::DeviceManagement.url_for_device(@device.merge("arguments" => {"a" => "b", "c" => "d"}))).to eq("ssh://rip:secret@1.2.3.4?a=b&c=d")
      expect(ASM::DeviceManagement.url_for_device(@device.merge(
                                                    "port" => 443,
                                                    "path" => "rspec/rspec",
                                                    "arguments" => {"a" => "b", "c" => "d"}
      ))).to eq("ssh://rip:secret@1.2.3.4:443/rspec/rspec?a=b&c=d")
    end

    it "should ensure all the paramaters are present" do
      expect do
        ASM::DeviceManagement.write_device_config({})
      end.to raise_error("Devices need cert_name, host, provider, scheme parameters")
    end

    it "should support custom device files" do
      ASM::DeviceManagement.expects(:device_config_name).never

      out = ASM::DeviceManagement.write_device_config(@device, true, @tmpfile.path)

      expect(out).to eq(@tmpfile.read)
    end

    it "should not overwrite files by default" do
      expect do
        ASM::DeviceManagement.write_device_config(@device, false, @test_dir)
      end.to raise_error(/Device file .* already exist/)
    end

    it "should write and return the file contents" do
      ASM::DeviceManagement.expects(:device_config_name).returns(@tmpfile.path)

      out = ASM::DeviceManagement.write_device_config(@device, true)

      expect(out).to eq(@tmpfile.read)
    end

    it "should have a overwrite short cut method" do
      ASM::DeviceManagement.expects(:write_device_config).with(@device, true, "/tmp")
      ASM::DeviceManagement.write_device_config!(@device, "/tmp")
    end
  end

  describe "when doing device discovery" do
    before do
      ASM::Util.stubs(:run_command_simple)
    end

    it "should determine the correct state dir" do
      expect(ASM::DeviceManagement.device_state_dir("rspec")).to eq("/var/opt/lib/pe-puppet/devices/rspec/state")
    end

    it "should determine the correct summary file" do
      expect(ASM::DeviceManagement.device_summary_file("rspec")).to eq("/var/opt/lib/pe-puppet/devices/rspec/state/last_run_summary.yaml")
    end

    it "should give a really old mtime for the nonexisting summary files" do
      ASM::DeviceManagement.expects(:device_summary_file).returns("/nonexisting")
      expect(ASM::DeviceManagement.last_run_summary_mtime("rspec")).to eq(Time.at(0))
    end

    it "should return the correct mtime for the summary files" do
      now = Time.now
      File.expects(:mtime).with(@test_dir).returns(now)
      ASM::DeviceManagement.stubs(:device_summary_file).returns(@test_dir)

      expect(ASM::DeviceManagement.last_run_summary_mtime("rspec")).to eq(now)
    end

    describe "when testing a logfile for pluginsync errors" do
      it "should return false if the logfile does not exist" do
        expect(ASM::DeviceManagement.log_has_pluginsync_errors?("/nonexisting")).to eq(false)
      end

      it "should return false if there are no pluginsync errors" do
        log = SpecHelper.fixture_path("pluginsync-passed-device-run")
        expect(ASM::DeviceManagement.log_has_pluginsync_errors?(log)).to eq(false)
      end

      it "should return true when there are pluginsync errors" do
        log = SpecHelper.fixture_path("pluginsync-failed-device-run")
        expect(ASM::DeviceManagement.log_has_pluginsync_errors?(log)).to eq(true)
      end

      it "should handle encoding errors gracefully" do
        log = SpecHelper.fixture_path("pluginsync-failed-device-run-wrong-encoding")
        expect(ASM::DeviceManagement.log_has_pluginsync_errors?(log)).to eq(true)
      end
    end

    describe "when testing a log file for dependency cycle error" do
      it "should return true if the logfile has dependecy cycle Error" do
        log = SpecHelper.fixture_path("dependency-cycle-exist")
        expect(ASM::DeviceManagement.log_has_dependency_cycle?(log)).to eq(true)
      end

      it "should return false if the logfile has no dependency cycle error" do
        log = SpecHelper.fixture_path("dependency-cycle-not-exist")
        expect(ASM::DeviceManagement.log_has_dependency_cycle?(log)).to eq(false)
      end

      it "should return false if the logfile does not exist" do
        expect(ASM::DeviceManagement.log_has_dependency_cycle?("/nonexisting")).to eq(false)
      end
    end

    describe "when determining the device run failed" do
      before do
        @result = Hashie::Mash.new
        @result.exit_status = 0
      end

      it "should fail when the exit code was !0" do
        @result.exit_status = 1

        expect(ASM::DeviceManagement.puppet_run_success?("rspec", @result, Time.now)).to eq([false, "puppet exit code was 1"])
      end

      it "should support numeric result codes" do
        expect(ASM::DeviceManagement.puppet_run_success?("rspec", 1, Time.now)).to eq([false, "puppet exit code was 1"])
      end

      it "should fail when the last_run_summary was not updated" do
        ASM::DeviceManagement.expects(:last_run_summary_mtime).with("rspec", nil).returns(Time.at(0))
        expect(ASM::DeviceManagement.puppet_run_success?("rspec", @result, Time.now)).to eq(
          [false, "/var/opt/lib/pe-puppet/devices/rspec/state/last_run_summary.yaml has not been updated in this run"]
        )
      end

      it "should fail when the last_run_summary is not valid yaml" do
        ASM::DeviceManagement.expects(:last_run_summary_mtime).with("rspec", nil).returns(Time.at(4102466400))
        ASM::DeviceManagement.expects(:last_run_summary).with("rspec", nil).raises "simulated error"
        expect(ASM::DeviceManagement.puppet_run_success?("rspec", @result, Time.now)).to eq(
          [false, "Could not parse /var/opt/lib/pe-puppet/devices/rspec/state/last_run_summary.yaml as YAML: #<RuntimeError: simulated error>"]
        )
      end

      it "should fail on non hash data" do
        ASM::DeviceManagement.expects(:last_run_summary_mtime).with("rspec", nil).returns(Time.at(4102466400))
        ASM::DeviceManagement.expects(:last_run_summary).with("rspec", nil).returns("")
        expect(ASM::DeviceManagement.puppet_run_success?("rspec", @result, Time.now)).to eq(
          [false, "/var/opt/lib/pe-puppet/devices/rspec/state/last_run_summary.yaml was not a hash"]
        )
      end

      it "should fail on empty data" do
        ASM::DeviceManagement.expects(:last_run_summary_mtime).with("rspec", nil).returns(Time.at(4102466400))
        ASM::DeviceManagement.expects(:last_run_summary).with("rspec", nil).returns({})
        expect(ASM::DeviceManagement.puppet_run_success?("rspec", @result, Time.now)).to eq(
          [false, "/var/opt/lib/pe-puppet/devices/rspec/state/last_run_summary.yaml is empty"]
        )
      end

      it "should fail when the config version is not valid" do
        ASM::DeviceManagement.expects(:last_run_summary_mtime).with("rspec", nil).returns(Time.at(4102466400))
        ASM::DeviceManagement.expects(:last_run_summary).with("rspec", nil).returns("version" => {"config" => nil})
        expect(ASM::DeviceManagement.puppet_run_success?("rspec", @result, Time.now)).to eq(
          [false, "/var/opt/lib/pe-puppet/devices/rspec/state/last_run_summary.yaml has an invalid config version"]
        )
      end

      it "should fail when there are no resources section" do
        ASM::DeviceManagement.expects(:last_run_summary_mtime).with("rspec", nil).returns(Time.at(4102466400))
        ASM::DeviceManagement.expects(:last_run_summary).with("rspec", nil).returns("version" => {"config" => 1})
        expect(ASM::DeviceManagement.puppet_run_success?("rspec", @result, Time.now)).to eq(
          [false, "/var/opt/lib/pe-puppet/devices/rspec/state/last_run_summary.yaml has no resources section"]
        )
      end

      it "should fail when there are pluginsync errors in the log" do
        ASM::DeviceManagement.expects(:last_run_summary_mtime).with("rspec", nil).returns(Time.at(4102466400))
        ASM::DeviceManagement.expects(:last_run_summary).with("rspec", nil).returns("version" => {"config" => 1}, "resources" => {})
        ASM::DeviceManagement.expects(:log_has_pluginsync_errors?).with("/some/file").returns(true)
        expect(ASM::DeviceManagement.puppet_run_success?("rspec", @result, Time.now, "/some/file")).to eq(
          [false, "pluginsync failed based on logfile /some/file"]
        )
      end

      it "should passs on good files and exit codes" do
        ASM::DeviceManagement.expects(:last_run_summary_mtime).with("rspec", nil).returns(Time.at(4102466400))
        ASM::DeviceManagement.expects(:last_run_summary).with("rspec", nil).returns("version" => {"config" => 1}, "resources" => {})
        expect(ASM::DeviceManagement.puppet_run_success?("rspec", @result, Time.now)).to eq(
          [true, "last_run_summary.yaml and exit code checks pass"]
        )
      end

      it "should support checking time for summary files" do
        ASM::DeviceManagement.expects(:last_run_summary_mtime).with("rspec", "/some/file").returns(Time.at(0))
        expect(ASM::DeviceManagement.puppet_run_success?("rspec", @result, Time.now, nil, "/some/file")).to eq(
          [false, "/some/file has not been updated in this run"]
        )
      end

      it "should support loading data from custom summary data files" do
        ASM::DeviceManagement.expects(:last_run_summary_mtime).with("rspec", "/some/file").returns(Time.at(4102466400))
        ASM::DeviceManagement.expects(:last_run_summary).with("rspec", "/some/file").returns("version" => {"config" => 1}, "resources" => {})
        expect(ASM::DeviceManagement.puppet_run_success?("rspec", @result, Time.now, nil, "/some/file")).to eq(
          [true, "last_run_summary.yaml and exit code checks pass"]
        )
      end

      it "should fail gracefully for any exceptions" do
        ASM::DeviceManagement.expects(:last_run_summary_mtime).with("rspec", nil).raises("simulated error")

        success, reason = ASM::DeviceManagement.puppet_run_success?("rspec", @result, Time.now)

        expect(success).to eq(false)
        expect(reason).to match(/Failed to determine puppet run success: simulated error at /)
      end
    end

    describe "when multiple discovery requests for the same device are made" do
      it "should not retry if no exception is raised" do
        logger = mock
        ASM::DeviceManagement.expects(:init_device_state).with("rspec", logger)
        ASM::DeviceManagement.expects(:run_puppet_device_sync!).with("rspec", logger)

        ASM::DeviceManagement.run_puppet_device!("rspec", logger)
      end

      it "should raise and rescue exception for device in progress discovery to finish" do
        logger = mock
        logger.expects(:info).at_least_once
        logger.expects(:info).with("Discovery for device rspec is already in progress - state is requested")
        ASM::PrivateUtil.expects(:facts_find).returns("cert_name" => "rspec", "update_time" => "2066-10-13 21:42:30 UTC")
        ASM::DeviceManagement.set_device_state("rspec", :requested)
        ASM::DeviceManagement.run_puppet_device!("rspec", logger)

        expect { ASM::DeviceManagement.init_device_state("rspec", logger) }.to raise_error(ASM::DeviceManagement::SyncException)
      end

      it "should retry if init_device_state raises an synchronous exception" do
        logger = mock
        logger.expects(:info).at_least_once

        ASM::DeviceManagement.expects(:sleep)
        ASM::DeviceManagement.expects(:init_device_state).with("rspec", logger)
        ASM::DeviceManagement.expects(:init_device_state).with("rspec", logger).raises(ASM::DeviceManagement::SyncException)
        ASM::DeviceManagement.expects(:run_puppet_device_sync!).with("rspec", logger)
        ASM::PrivateUtil.expects(:facts_find).returns("cert_name" => "rspec", "update_time" => "2010-10-13 21:42:30 UTC")
        ASM::DeviceManagement.run_puppet_device!("rspec", logger)
      end

      it "should retry  run_puppet_device_sync! if inventory is not updated successfully" do
        logger = mock
        ASM::DeviceManagement.set_device_state("rspec", :success)
        logger.expects(:info).at_least_once
        ASM::DeviceManagement.expects(:sleep).at_least_once
        ASM::DeviceManagement.expects(:get_device_state).with("rspec").at_least_once.returns(:success)
        ASM::PrivateUtil.expects(:facts_find).returns("cert_name" => "rspec", "update_time" => "2066-10-13 21:42:30 UTC")
        ASM::PrivateUtil.expects(:facts_find).returns("cert_name" => "rspec", "update_time" => "2016-10-10 21:42:30 UTC")
        ASM::DeviceManagement.expects(:run_puppet_device_sync!).with("rspec", logger).twice.raises(ASM::DeviceManagement::SyncException)

        ASM::DeviceManagement.run_puppet_device!("rspec", logger)
        expect(ASM::DeviceManagement.get_device_state("rspec")).to eq(:success)
      end

      it "should raise synchronous exception after certain timeout" do
        logger = mock
        logger.expects(:info).at_least_once
        ASM::DeviceManagement.expects(:init_device_state).with("rspec", logger).at_least_once.raises(ASM::DeviceManagement::SyncException)
        Time.expects(:now).times(2).returns(Time.new(2099))
        Time.expects(:now).returns(Time.new(2055))
        ASM::PrivateUtil.expects(:facts_find).at_least_once.returns("cert_name" => "rspec", "update_time" => "2010-10-13 21:42:30 UTC")
        expect { ASM::DeviceManagement.run_puppet_device!("rspec", logger) }.to raise_error("Timed out waiting to initiate new inventory deployment for rspec")
      end
    end

    it "should handle any exception as a failure" do
      logger = mock
      logger.expects(:info).with("Puppet device run for node rspec caught exception: rspec error")

      ASM::DeviceManagement.set_device_state("rspec", :success)
      ASM::PrivateUtil.expects(:wait_until_available).raises("rspec error")
      expect do
        ASM::DeviceManagement.run_puppet_device!("rspec", logger)
      end.to raise_error("rspec error")
      expect(ASM::DeviceManagement.get_device_state("rspec")).to eq(:failed)
    end

    it "should handle no device config as a failure" do
      logger = mock
      logger.expects(:info).with("No device config available for rspec")

      ASM::DeviceManagement.set_device_state("rspec", :success)
      ASM::DeviceManagement.run_puppet_device!("rspec", logger)
      expect(ASM::DeviceManagement.get_device_state("rspec")).to eq(:failed)
    end

    it "should interpret puppet command failures correctly" do
      result = Hashie::Mash.new
      result.exit_status = 1
      logger = mock
      ASM::DeviceManagement.stubs(:parse_device_config).with("rspec").returns("rspec" => true)

      logger.expects(:info).with("Puppet device run for node rspec started, logging to /tmp/log")
      logger.expects(:info).with("Puppet device run for node rspec failed in 0.00 seconds: puppet exit code was 1")

      ASM::Util.expects(:run_command_simple).returns(result)
      ASM::DeviceManagement.expects(:device_log_file).returns("/tmp/log")

      ASM::DeviceManagement.set_device_state("rspec", :success)
      ASM::DeviceManagement.run_puppet_device!("rspec", logger)
      expect(ASM::DeviceManagement.get_device_state("rspec")).to eq(:failed)
    end

    it "should interpret puppet command success correctly" do
      result = Hashie::Mash.new
      result.exit_status = 0
      logger = mock
      ASM::DeviceManagement.stubs(:parse_device_config).with("rspec").returns("rspec" => true)

      ASM::Util.expects(:run_command_simple).returns(result)
      ASM::DeviceManagement.expects(:device_log_file).returns("/tmp/log")
      ASM::DeviceManagement.expects(:puppet_run_success?).returns([true, ""])

      logger.expects(:info).with("Puppet device run for node rspec started, logging to /tmp/log")
      logger.expects(:info).with("Puppet device run for node rspec succeeded in 0.00 seconds")

      ASM::DeviceManagement.set_device_state("rspec", :failed)
      ASM::DeviceManagement.run_puppet_device!("rspec", logger)
      expect(ASM::DeviceManagement.get_device_state("rspec")).to eq(:success)
    end
  end

  describe "when loading config files" do
    before do
      File.stubs(:exist?).with(ASM::DeviceManagement.device_config_name("rspec")).returns(true)
      ASM::DeviceManagement.stubs(:parse_device_config).with("rspec").returns("rspec" => true)
      puppetdb = mock("puppetdb")
      puppetdb.stubs(:facts).with("rspec").returns(:rspec => :facts)
      ASM::Client::Puppetdb.stubs(:new).returns(puppetdb)
      ASM::DeviceManagement.stubs(:get_device_state).returns(:rspec)
    end

    it "should detect unknown devices" do
      File.stubs(:exist?).with(ASM::DeviceManagement.device_config_name("rspec")).returns(false)
      expect { ASM::DeviceManagement.get_device("rspec") }.to raise_error("Device rspec is unknown")
    end

    it "should support loading a device configuration and default to not supplying facts" do
      ASM::PrivateUtil.expects(:facts_find).never
      expect(ASM::DeviceManagement.get_device("rspec", false)).to eq("rspec" => true, "facts" => {}, "discovery_status" => :rspec)
    end

    it "should support loading facts for a device configuration" do
      expect(ASM::DeviceManagement.get_device("rspec")).to eq("rspec" => true, "facts" => {:rspec => :facts}, "discovery_status" => :rspec)
    end

    it "should set the discovery status to unknown when facts arent requested and no status is known" do
      ASM::DeviceManagement.expects(:get_device_state).returns(:unknown)
      expect(ASM::DeviceManagement.get_device("rspec", false)).to eq("rspec" => true, "facts" => {}, "discovery_status" => :unknown)
    end

    it "should set the discovery status to success when there are facts and no status is known" do
      ASM::DeviceManagement.expects(:get_device_state).returns(:unknown)
      expect(ASM::DeviceManagement.get_device("rspec")).to eq("rspec" => true, "facts" => {:rspec => :facts}, "discovery_status" => :success)
    end
  end
end
