require "spec_helper" # rubocop:disable Style/FileName

$LOAD_PATH << File.expand_path(File.join(File.dirname(__FILE__), "..", "..", "..", "..", "nagios"))

require "check-racadm"

describe "When checking racadm devices" do
  let(:options) do
    {:host => "rspec",
     :port => 22,
     :user => "root",
     :password => "calvin",
     :module_svctag => nil,
     :module_slot => nil,
     :check_power => nil,
     :decrypt => false}
  end
  let(:check) { ASM::Nagios::CheckRacadm.new(options) }

  before(:each) do
    # Set activeerrors to empty.
    check.racadm_getactiveerrors("")
  end

  it "should fail to parse with correct return values" do
    expect(check.racadm_getmodinfo("rspec")).to eq("asm_error" => "Could not parse output from racadm, unexpected line: rspec", "asm_nagios_code" => 3)
    expect(check.racadm_getmodinfo("")).to eq("asm_error" => "Could not parse output from racadm, no output were received", "asm_nagios_code" => 3)
  end

  it "should support an all green m1000e chassis" do
    stub_fixture = SpecHelper.load_fixture("racadm/m1000e_chassis_all_green")
    status = check.racadm_getmodinfo(stub_fixture)
    expect(status.size).to eq(43)
    expect(check.get_status_from_modinfo(status)).to eq([0, "OK"])
  end

  it "should support an all green fx2 chassis" do
    stub_fixture = SpecHelper.load_fixture("racadm/fx2_chassis_all_green")
    status = check.racadm_getmodinfo(stub_fixture)
    expect(status.size).to eq(21)
    expect(check.get_status_from_modinfo(status)).to eq([0, "OK"])
  end

  it "should detect a single module as critical when checking the whole chassis" do
    stub_fixture = SpecHelper.load_fixture("racadm/fx2_chassis_server-2_critical")
    status = check.get_status_from_modinfo(check.racadm_getmodinfo(stub_fixture), nil, nil)

    expect(status).to eq([1, "Server-2: Degraded"])
  end

  it "should detect a single module as critical when checking that module by module name" do
    stub_fixture = SpecHelper.load_fixture("racadm/fx2_chassis_server-2_critical")
    status = check.get_status_from_modinfo(check.racadm_getmodinfo(stub_fixture), nil, "Server-2")

    expect(status).to eq([1, "Server-2: Degraded"])
  end

  it "should detect a single module as critical when checking that module by svctag" do
    stub_fixture = SpecHelper.load_fixture("racadm/fx2_chassis_server-2_critical")
    status = check.get_status_from_modinfo(check.racadm_getmodinfo(stub_fixture), "ASMFC02", nil)

    expect(status).to eq([1, "Server-2: Degraded"])
  end

  it "should detect a single module as OK when checking that module by slot and another module is critical" do
    stub_fixture = SpecHelper.load_fixture("racadm/fx2_chassis_server-2_critical")
    status = check.get_status_from_modinfo(check.racadm_getmodinfo(stub_fixture), nil, "Switch-1")

    expect(status).to eq([0, "OK"])
  end

  it "should detect a single module as OK when checking that module by svctag and another module is critical" do
    stub_fixture = SpecHelper.load_fixture("racadm/fx2_chassis_server-2_critical")
    status = check.get_status_from_modinfo(check.racadm_getmodinfo(stub_fixture), "H4YRTS5", nil)

    expect(status).to eq([0, "OK"])
  end

  it "should not detect powered off switches as critical when not checking power" do
    stub_fixture = SpecHelper.load_fixture("racadm/fx2_chassis_switch_power_off")
    status = check.get_status_from_modinfo(check.racadm_getmodinfo(stub_fixture), nil, "Switch-2")

    expect(status).to eq([0, "OK"])
  end

  it "should detect a switch being off as critical when checking power" do
    check.options[:check_power] = true

    stub_fixture = SpecHelper.load_fixture("racadm/fx2_chassis_switch_power_off")
    status = check.get_status_from_modinfo(check.racadm_getmodinfo(stub_fixture), nil, "Switch-2")

    expect(status).to eq([2, "Switch-2: Error"])
  end

  it "should parse corrupt power errors correctly" do
    stub_fixture = SpecHelper.load_fixture("racadm/m1000e_chassis_showing_power_error")
    modinfo = check.racadm_getmodinfo(stub_fixture)
    expect(modinfo.size).to eq(43)
    expect(modinfo["PS-5"]).to eq("presence" => "Present", "pwrState" => "Failed(No Input Power)", "svcTag" => "", "health" => 1)
    expect(modinfo["PS-6"]).to eq("presence" => "Present", "pwrState" => "Failed(No Input Power)", "svcTag" => "N/A", "health" => 1)
  end

  it "should correctly parse racadm output with blank svctags" do
    stub_fixture = SpecHelper.load_fixture("racadm/m1000e_chassis_svctags_blank")
    modinfo = check.racadm_getmodinfo(stub_fixture)

    expect(modinfo.size).to eq(43)
    expect(modinfo["Fan-1"]).to eq("presence" => "Present", "pwrState" => "ON", "svcTag" => "", "health" => 0)
    expect(modinfo["Switch-1"]).to eq("presence" => "Present", "pwrState" => "ON", "svcTag" => "NAVA100", "health" => 0)
  end

  it "should detect versions of racadm without getmodinfo" do
    stub_fixture = SpecHelper.load_fixture("racadm/m1000e_chassis_no_getmodinfo")
    modinfo = check.racadm_getmodinfo(stub_fixture)
    expect(modinfo["asm_error"]).to eq("Host rspec does not support get the getmodinfo command")
    expect(modinfo["asm_nagios_code"]).to eq(3)
  end

  it "should correctly parse activeerrors output" do
    stub_fixture = SpecHelper.load_fixture("racadm/m1000e_chassis_activeerrors")
    errors = check.racadm_getactiveerrors(stub_fixture)

    # ae.size.should == 43
    expect(errors["server-14"]).to eq("CPU 1 machine check error detected.")
  end
end
