require "spec_helper"
require "open3"

describe "When short circuiting execution" do
  before do
    skip("Skipping bash tests on Windows") if /mswin|mingw/ =~ RUBY_PLATFORM
  end

  it "should prevent execution when load is too high" do
    o, _, s = Open3.capture3("bash nagios/load_short_circuit.sh -f %s -c 'echo command ran'" % SpecHelper.fixture_path("nagios/load_3_0"))

    expect(s.exitstatus).to eq(0)
    expect(o.chomp).to eq("Skipping the run due to load average 3.0 being greater than 2")
  end

  it "should allow execution when load is below the threshold" do
    o, _, s = Open3.capture3("bash nagios/load_short_circuit.sh -f %s -c 'echo command ran'" % SpecHelper.fixture_path("nagios/load_1_5"))

    expect(s.exitstatus).to eq(0)
    expect(o.chomp).to eq("command ran")
  end

  it "should fail on non integer and float load averages for comparison" do
    ["a", "1.1.1"].each do |l|
      _, e, s = Open3.capture3("bash nagios/load_short_circuit.sh -l %s" % l)
      expect(e.chomp).to eq("The load average specified in -l should be a integer or float")
      expect(s.exitstatus).to eq(1)
    end
  end

  it "should fail on non integer exit codes" do
    ["a", "1.1"].each do |l|
      _, e, s = Open3.capture3("bash nagios/load_short_circuit.sh -e %s" % l)
      expect(e.chomp).to eq("The exit code specified in -e should be a integer")
      expect(s.exitstatus).to eq(1)
    end
  end

  it "should fail when the load average source is absent" do
    _, e, s = Open3.capture3("bash nagios/load_short_circuit.sh -f /nonexistant")
    expect(e.chomp).to eq("The load average file /nonexistant cannot be found")
    expect(s.exitstatus).to eq(1)
  end

  it "should support a default off debug output" do
    _, e, = Open3.capture3("bash nagios/load_short_circuit.sh -v")
    expect(e).to match(/System load sourced from \/proc\/loadavg is \d+\.\d+ and will be checked against a maximum load of 2/m)

    _, e, = Open3.capture3("bash nagios/load_short_circuit.sh")
    expect(e).not_to match(/System load sourced from/m)
  end

  it "should support surpressing all non error output" do
    o, = Open3.capture3("bash nagios/load_short_circuit.sh -f %s -c 'echo command ran' -s" % SpecHelper.fixture_path("nagios/load_3_0"))

    expect(o).not_to match(/Skipping the run due to load average 3.0 being greater than 2/)
  end

  it "should support a custom skip message" do
    o, _, s = Open3.capture3("bash nagios/load_short_circuit.sh -f %s -c 'echo command ran' -m 'skipped by rspec'" % SpecHelper.fixture_path("nagios/load_3_0"))

    expect(s.exitstatus).to eq(0)
    expect(o.chomp).to eq("skipped by rspec")
    expect(o).not_to match(/command ran/m)
  end

  it "should support a custom exit code when skipping" do
    o, _, s = Open3.capture3("bash nagios/load_short_circuit.sh -f %s -c 'echo command ran' -e 100" % SpecHelper.fixture_path("nagios/load_3_0"))

    expect(s.exitstatus).to eq(100)
    expect(o.chomp).to eq("Skipping the run due to load average 3.0 being greater than 2")
  end

  it "should support not running a command on success instead just exiting 0" do
    _, e, s = Open3.capture3("bash nagios/load_short_circuit.sh -f %s -v" % SpecHelper.fixture_path("nagios/load_1_5"))
    expect(s.exitstatus).to eq(0)
    expect(e).to match(/No command given so exiting 0 while load below 2/m)
  end

  it "should support showing help" do
    o, _, s = Open3.capture3("bash nagios/load_short_circuit.sh -h")
    expect(s.exitstatus).to eq(1)
    expect(o).to match(/Only runs COMMAND when the system load average is below LOAD/m)
  end
end
