require "spec_helper"
require "asm"

module ASM
  describe PortView do
    let(:network_overview_data) { Hashie::Mash.new(SpecHelper.json_fixture("network_overview.json")) }
    let(:network_overview_with_extra_data) { Hashie::Mash.new(SpecHelper.json_fixture("network_overview_with_extra_data.json")) }
    let(:logger) { stub(:info => nil, :debug => nil) }
    let(:server) { stub(:puppet_certname => "rspec.name") }
    let(:portview) { ASM::PortView.new(server, logger) }

    describe "#deployment_json_files" do
      it "should sort the deployments ascending by ctime" do
        files = [
          "/opt/Dell/ASM/deployments/rspecid/deployment.json",
          "/opt/Dell/ASM/deployments/rspecid/deployment__1.json",
          "/opt/Dell/ASM/deployments/rspecid/deployment__2.json",
          "/opt/Dell/ASM/deployments/rspecid/deployment__3.json"
        ]

        ASM.stubs(:base_dir).returns("/nonexisting")
        Dir.expects(:glob).with("/nonexisting/rspecid/deployment*.json").returns(files.reverse)

        files.each_with_index do |file, idx|
          File.expects(:ctime).with(file).returns(Time.at(idx))
        end

        expect(portview.deployment_json_files("rspecid")).to eq(files)
      end
    end

    describe "#delete_cache" do
      it "should delete the cache but only if it exists" do
        File.expects(:exist?).with("/opt/Dell/ASM/cache/rspec.name_portview.json").returns(true)
        File.expects(:unlink).with("/opt/Dell/ASM/cache/rspec.name_portview.json").once
        portview.delete_cache

        File.expects(:exist?).with("/opt/Dell/ASM/cache/rspec.name_portview.json").returns(false)
        File.expects(:unlink).with("/opt/Dell/ASM/cache/rspec.name_portview.json").never

        server.stubs(:puppet_certname).returns("rspec.NAME")
        portview.delete_cache
      end
    end

    describe "#deployment_data" do
      it "should attempt a few times to retrieve the data" do
        portview.expects(:file_absent_or_zero?).with("/nonexisting/deployment.json").returns(true, true, false).times(3)

        portview.stubs(:deployment_json_files).returns(["/nonexisting/1/deployment.json", "/nonexisting/deployment.json"])

        File.expects(:read).with("/nonexisting/deployment.json").returns('{"rspec":1}')

        portview.expects(:sleep).with(1).twice

        expect(portview.deployment_data("x")).to eq("rspec" => 1)
      end

      it "should return nil after all retries if the file does not exist" do
        portview.stubs(:file_absent_or_zero?).with("/nonexisting/deployment.json").returns(true)

        portview.stubs(:deployment_json_files).returns(["/nonexisting/1/deployment.json", "/nonexisting/deployment.json"])

        File.expects(:read).never

        portview.expects(:sleep).with(1).times(9)

        expect(portview.deployment_data("x")).to be_nil
      end
    end

    describe "#file_absent_or_zero?" do
      it "should handle nil files" do
        expect(portview.file_absent_or_zero?(nil)).to be_truthy
      end

      it "should correctly detect absent files" do
        File.expects(:exist?).with("/nonexisting").returns(false)
        File.expects(:zero?).never

        expect(portview.file_absent_or_zero?("/nonexisting")).to be_truthy
      end

      it "should correctly detect zero size files" do
        File.expects(:exist?).with("/nonexisting").returns(true)
        File.expects(:zero?).with("/nonexisting").returns(true)

        expect(portview.file_absent_or_zero?("/nonexisting")).to be_truthy
      end

      it "should correctly report existing non zero byte files" do
        expect(portview.file_absent_or_zero?(__FILE__)).to be_falsey
      end
    end

    describe "has_cache?" do
      it "should detect valid caches" do
        File.stubs(:exist?).with("/opt/Dell/ASM/cache/rspec.name_portview.json").returns(true)
        File.stubs(:file?).with("/opt/Dell/ASM/cache/rspec.name_portview.json").returns(true)
        File.stubs(:symlink?).with("/opt/Dell/ASM/cache/rspec.name_portview.json").returns(false)
        File.stubs(:zero?).with("/opt/Dell/ASM/cache/rspec.name_portview.json").returns(false)
        File.stubs(:readable?).with("/opt/Dell/ASM/cache/rspec.name_portview.json").returns(true)
        expect(portview.has_cache?).to be_truthy

        File.expects(:exist?).with("/opt/Dell/ASM/cache/rspec.name_portview.json").returns(false)
        expect(portview.has_cache?).to be_falsey

        File.expects(:file?).with("/opt/Dell/ASM/cache/rspec.name_portview.json").returns(false)
        expect(portview.has_cache?).to be_falsey

        File.expects(:symlink?).with("/opt/Dell/ASM/cache/rspec.name_portview.json").returns(true)
        expect(portview.has_cache?).to be_falsey

        File.expects(:zero?).with("/opt/Dell/ASM/cache/rspec.name_portview.json").returns(true)
        expect(portview.has_cache?).to be_falsey

        File.expects(:readable?).with("/opt/Dell/ASM/cache/rspec.name_portview.json").returns(false)
        expect(portview.has_cache?).to be_falsey
      end
    end

    describe "#write_cache" do
      it "should write the right data" do
        portview.expects(:updated_port_view_data).returns("rspec" => "updated", "DATA_VERSION" => 1)
        File.expects(:write).with("/opt/Dell/ASM/cache/rspec.name_portview.json", {"rspec" => "updated", "DATA_VERSION" => 1}.to_json)
        expect(portview.write_cache).to eq("rspec" => "updated", "DATA_VERSION" => 1)
      end
    end

    describe "#cache_filename" do
      it "should determine the right names" do
        expect(portview.cache_filename).to eq("/opt/Dell/ASM/cache/rspec.name_portview.json")
        server.stubs(:puppet_certname).returns("rspec.NAME")
        expect(portview.cache_filename).to eq("/opt/Dell/ASM/cache/rspec.name_portview.json")
      end
    end

    describe "#network_overview" do
      it "should return nil when no server is given" do
        expect(PortView.new(nil, logger).network_overview).to be_nil
      end

      it "should return the correct data" do
        expected_pv = SpecHelper.json_fixture("network_overview.json", :symbolize_names => true)
        server_pv = SpecHelper.json_fixture("network_overview.json", :symbolize_names => true)

        server_pv[:extra] = 1

        server.expects(:network_overview).returns(server_pv)

        expect(portview.network_overview).to eq(expected_pv)
      end
    end

    describe "#server_info" do
      it "should gracefully handle absent deployment" do
        portview.expects(:deployment_data).with("rspec_id").returns(nil)
        Service.expects(:new).never

        expect(portview.server_info("rspec_id", "rspec_node")).to eq({})
      end

      it "should gracefully handle absent deployment" do
        portview.expects(:deployment_data).with("rspec_id").returns({})
        portview.expects(:deployment_service).returns(service = stub)
        service.expects(:servers).returns([])
        portview.expects(:read_cache).never

        expect(portview.server_info("rspec_id", "rspec_2")).to eq({})
      end

      it "should return the data for the correct servers" do
        servers = [
          stub(:puppet_certname => "rspec_1"),
          stub(:puppet_certname => "rspec_2", :network_overview => network_overview_data)
        ]

        portview.expects(:deployment_data).with("rspec_id").returns({})
        portview.expects(:deployment_service).returns(service = stub)
        service.expects(:servers).returns(servers)
        portview.expects(:read_cache)

        portview.server_info("rspec_id", "rspec_2")

        expect(portview.server).to be(servers[1])
      end
    end

    describe "#updated_port_view_data" do
      it "should not alter correct network overview" do
        portview.expects(:network_overview).returns(network_overview_data)
        expect(portview.updated_port_view_data).to eq(network_overview_data.merge("DATA_VERSION" => 1))
      end

      it "should generate correct network overview for specified keys" do
        portview.expects(:network_overview).returns(network_overview_with_extra_data)
        expect(portview.updated_port_view_data).to eq(network_overview_data.merge("DATA_VERSION" => 1))
      end
    end
  end
end
