require "spec_helper"
require "asm/processor/win_post_os"
require "asm/processor/windows_vm_post_os"
require "asm/private_util"
require "asm/service"
require "asm/resource"

describe ASM::Processor::WindowsVMPostOS do
  let(:puppetdb) { double(:successful_report_after? => true) }

  before do
    @service_deployment = SpecHelper.json_fixture("processor/win_post_os/win_vm_post_os.json")
    @sd = mock("service_deployment")
    @sd = ASM::ServiceDeployment.new("8000", nil)
    @sd.stubs(:logger).returns(stub(:debug => nil, :warn => nil, :info => nil))
    @sd.components(@service_deployment)

    service = ASM::Service.new(@service_deployment, :deployment => @sd)

    @resource1 = service.resources_by_type("VIRTUALMACHINE").first
    @resource1.stubs(:macaddress).returns("aa-bb-cc-dd-ee-ff")

    @processor1 = ASM::Processor::WindowsVMPostOS.new(@resource1)

    resource2 = service.resources_by_type("VIRTUALMACHINE")[1]
    resource2.stubs(:macaddress).returns("aa-bb-cc-dd-ee-ff")
    @processor2 = ASM::Processor::WindowsVMPostOS.new(resource2)
  end

  describe "should configure windows vm post installation" do
    it "with no static network adapter" do
      @processor1.stubs(:nic_ip_config).returns({})
      expect(@processor1.post_os_classes).to eql("windows_postinstall::cleanup::cleanup" => {})
    end

    it "with one static network adapter" do
      static_network = {"windows_postinstall::nic::adapter_nic_ip_settings"=>
                         {"ipaddress_info"=>
                          {"NICIPInfo"=>
                           [{"adapter_name"=>"Workload-22-Static",
                             "ip_address"=>"172.22.10.65",
                             "subnet"=>"255.255.0.0",
                             "primaryDns"=>"172.20.0.8",
                             "dnsSuffix" => "asmdev.local",
                             "mac_address"=>"aa-bb-cc-dd-ee-ff",
                             "gateway"=>"172.22.0.1",
                             "adapter_type"=>"vm_network"}]}},
                        "windows_postinstall::cleanup::cleanup" => {}}

      expect(@processor1.post_os_classes).to eql(static_network)
    end

    it "with two static network adapters" do
      static_two_networks = {"windows_postinstall::nic::adapter_nic_ip_settings"=>
                              {"ipaddress_info"=>
                               {"NICIPInfo"=>
                                [{"adapter_name"=>"Workload-22-Static",
                                  "ip_address"=>"172.22.10.66",
                                  "subnet"=>"255.255.0.0",
                                  "primaryDns"=>"172.20.0.8",
                                  "dnsSuffix" => "asmdev.local",
                                  "mac_address"=>"aa-bb-cc-dd-ee-ff",
                                  "gateway"=>"172.22.0.1",
                                  "adapter_type"=>"vm_network"},
                                 {"adapter_name"=>"Workload-25-static",
                                  "ip_address"=>"172.25.10.64",
                                  "subnet"=>"255.255.0.0",
                                  "primaryDns"=>"172.20.0.8",
                                  "dnsSuffix" => "asmdev.local",
                                  "mac_address"=>"aa-bb-cc-dd-ee-ff",
                                  "adapter_type"=>"vm_network"}]}},
                             "windows_postinstall::cleanup::cleanup" => {}}
      expect(@processor2.post_os_classes).to eql(static_two_networks)
    end

    it "with no default gateway" do
      static_without_gateway =  {"windows_postinstall::nic::adapter_nic_ip_settings"=>
                                  {"ipaddress_info"=>
                                   {"NICIPInfo"=>
                                    [{"adapter_name"=>"Workload-22-Static",
                                      "ip_address"=>"172.22.10.65",
                                      "subnet"=>"255.255.0.0",
                                      "primaryDns"=>"172.20.0.8",
                                      "dnsSuffix" => "asmdev.local",
                                      "mac_address"=>"aa-bb-cc-dd-ee-ff",
                                      "adapter_type"=>"vm_network"}]}},
                                 "windows_postinstall::cleanup::cleanup" => {}}
      @resource1.stubs(:default_gateway).returns("")
      processor = ASM::Processor::WindowsVMPostOS.new(@resource1)
      expect(processor.post_os_classes).to eql(static_without_gateway)
    end


    it "with no static network" do
      dhcp_vm_network = [
          {"id"=>"ff80808150b4e04d0150b4f1394800a4",
           "name"=>"Workload-22-Static",
           "description"=>"",
           "type"=>"PUBLIC_LAN",
           "vlanId"=>20,
           "static"=>false,
           "staticNetworkConfiguration"=>{} }
      ]

      dhcp_nework = {"windows_postinstall::nic::adapter_nic_ip_settings"=>
                      {"ipaddress_info"=>
                       {"NICIPInfo"=>
                        [{"adapter_name"=>"Workload-22-Static",
                          "ip_address"=>"dhcp",
                          "subnet"=>"",
                          "gateway"=>"",
                          "primaryDns"=>"",
                          "mac_address"=>"aa-bb-cc-dd-ee-ff",
                          "adapter_type"=>"vm_network"}]}},
                     "windows_postinstall::cleanup::cleanup" => {}}
      @resource1.stubs(:network_interfaces).returns(dhcp_vm_network)
      @resource1.deployment.stubs(:bm_tagged?).returns(false)
      processor = ASM::Processor::WindowsVMPostOS.new(@resource1)
      expect(processor.post_os_classes).to eql(dhcp_nework)
    end

  end
end
