require "spec_helper"
require "asm/type"
require "asm/firmware"

ASM::Type.load_providers!

describe ASM::Provider::Configuration::Serverupdate do
  let(:logger) { stub(:debug => nil, :warn => nil, :info => nil) }
  let(:idrac_nfs_path) do
    [{"instance_id" => "DCIM:INSTALLED#701__NIC.Slot.2-2-1",
      "uri_path" => "nfs://172.25.5.100/firmware.exe;mountpoint=/rspec-nfs/fake/spec/path"
     },
     {"instance_id" => "DCIM:INSTALLED#701__NIC.Embedded.1-1-1",
      "uri_path" => "nfs://172.25.5.100/firmware.exe;mountpoint=/nfs-path/fake/spec/path"
     },
     {"instance_id" => "DCIM:INSTALLED#iDRAC.Embedded.1-1#IDRACinfo",
      "component_id" => "25227",
      "uri_path" => "nfs://172.25.5.100/firmware.exe;mountpoint=/rspec-nfs/fake/spec/path"
     }]
  end
  let(:service) { SpecHelper.service_from_fixture("server_update.json") }
  let(:server_update_components) { service.components.select { |c| c.resource_ids.include?("asm::server_update") } }
  let(:type) { server_update_components[0].to_resource(nil, logger) }
  let(:provider) { type.provider }

  describe "#server_firmware_update_munger" do
    it "should handle server_firmware correctly" do
      expect(provider.server_firmware).to eq(nil) # Because the fixture json indicates "[]" which is empty list
    end
  end

  describe "#server_software_update_munger" do
    it "should handle server_software correctly" do
      server_software_list = JSON.parse(provider.server_software)
      expect(server_software_list.empty?).to eq(false)
      # Try setting to empty list and verify nil value
      provider.server_software = "[]"
      expect(provider.server_software).to eq(nil)
    end
  end

  describe "#process!" do
    it "should update firmware through util" do
      provider.stubs(:requires_puppet?).returns(nil)
      provider.stubs(:to_puppet).returns("rspec-resources").once
      type.stubs(:device_config).returns("rspec-device-config")
      provider.stubs(:create_idrac_fw_nfs_path).returns("rspec-path")
      ASM::Firmware.expects(:idrac_fw_install_from_uri).with("rspec-resources", "rspec-path", "rspec-device-config", logger).returns(nil)
      type.expects(:update_inventory).never
      provider.process!
    end

    it "should update firmware through puppet process_genric" do
      provider.stubs(:requires_puppet?).returns("1")
      provider.stubs(:to_puppet).returns("rspec-resources").once
      type.stubs(:puppet_certname).returns("xyz")
      provider.stubs(:puppet_run_type).returns("apply")
      type.stubs(:guid).returns("123")
      type.expects(:process_generic).with("xyz", "rspec-resources", "apply", true, nil, "123", false)
      provider.process!
    end

    it "should update the inventory" do
      provider.stubs(:requires_puppet?).returns(nil)
      provider.stubs(:to_puppet).returns("rspec-resources").once
      type.stubs(:device_config).returns("rspec-device-config")
      provider.stubs(:create_idrac_fw_nfs_path).returns("rspec-nfs-path")
      ASM::Firmware.expects(:idrac_fw_install_from_uri).with("rspec-resources", "rspec-nfs-path", "rspec-device-config", logger)
      type.expects(:update_inventory)
      provider.process!(:update_inventory => true)
    end

    it "should invoke maintenance mode check" do
      provider.stubs(:requires_puppet?).returns("1")
      provider.update_property("vcenter_server_list", "[\"server1\",\"server2\"]")
      provider.update_property("vcenter_cert", "xyz")
      provider.stubs(:to_puppet).returns("rspec-resources").once
      type.stubs(:puppet_certname).returns("xyz")
      provider.stubs(:puppet_run_type).returns("apply")
      type.stubs(:guid).returns("123")
      type.expects(:process_generic).with("xyz", "rspec-resources", "apply", true, nil, "123", false)
      maintenance_mode_hash = {"esx_maintmode" =>
                                   {"server1" =>
                                        {"ensure" => "absent",
                                         "evacuate_powered_off_vms" => true,
                                         "fail_when_undetermined" => false,
                                         "timeout" => 0,
                                         "transport" => "Transport[vcenter]"},
                                    "server2" =>
                                        {"ensure" => "absent",
                                         "evacuate_powered_off_vms" => true,
                                         "fail_when_undetermined" => false,
                                         "timeout" => 0,
                                         "transport" => "Transport[vcenter]"}},
                               "transport" =>
                                   {"vcenter" =>
                                        {"name" => "xyz",
                                         "options" =>
                                             {"insecure" => true},
                                         "provider" => "device_file"}}}
      type.expects(:process_generic).with("xyz", maintenance_mode_hash, "apply", true)
      provider.process!
    end
  end

  describe "#create_idrac_fw_nfs_path" do
    let(:host) { "172.25.5.100" }
    let(:server_firmware) do
      "[{\"instance_id\":\"DCIM:INSTALLED#701__NIC.Slot.2-2-1\",\"uri_path\":\"/rspec-nfs/fake/spec/path/firmware.exe\"},
        {\"instance_id\":\"DCIM:INSTALLED#701__NIC.Embedded.1-1-1\",\"uri_path\":\"/nfs-path/fake/spec/path/firmware.exe\"},
        {\"instance_id\":\"DCIM:INSTALLED#iDRAC.Embedded.1-1#IDRACinfo\",\"component_id\":\"25227\",\"uri_path\":\"/rspec-nfs/fake/spec/path/firmware.exe\"}
       ]"
    end

    it "should provide valid mountbale nfs path" do
      provider.server_firmware = server_firmware
      provider.asm_hostname = host
      expect(provider.create_idrac_fw_nfs_path).to eq(idrac_nfs_path)
    end
  end
end
