require "spec_helper"
require "asm/type"

ASM::Type.load_providers!

describe ASM::Provider::Volume::Equallogic do
  let(:service) { SpecHelper.service_from_fixture("Teardown_EQL_VMware_Cluster.json") }
  let(:logger) { stub(:debug => nil, :warn => nil, :info => nil) }
  let(:deployment) { stub(:id => "1234", :is_teardown? => true, :debug? => false, :log => nil) }
  let(:volume_component) { service.components_by_type("STORAGE")[0] }
  let(:server_components) { service.components_by_type("SERVER") }
  let(:cluster_components) { service.components_by_type("CLUSTER") }

  let(:type) { volume_component.to_resource(deployment, logger) }
  let(:provider) { type.provider }

  let(:expected_esx_datastore) do
    {"esx_datastore" =>
      {"172.28.10.63:M830-01" =>
        {"ensure" => "absent",
         "datastore" => "M830-01",
         "type" => "vmfs",
         "target_iqn" => "RSPECIQN",
         "transport" => "Transport[vcenter]"
        }
      }
    }
  end

  before :each do
    ASM::NetworkConfiguration.any_instance.stubs(:add_nics!)
    ASM::PrivateUtil.stubs(:get_eql_volume_iqn).returns("RSPECIQN")
  end

  describe "#iscsi_target_name" do
    it "should fetch the right target name" do
      provider.facts = SpecHelper.json_fixture("equallogic-ess-rowc-rack7-eql6110-3_facts.json")
      provider.expects(:uuid).returns("Newasm")
      expect(provider.iscsi_target_name).to eq("iqn.2001-05.com.equallogic:0-1cb196-ee5154c2d-d8b000000165475f-newasm")
    end

    it "should handle absent volume properties" do
      provider.facts = SpecHelper.json_fixture("equallogic-ess-rowc-rack7-eql6110-3_facts.json")
      provider.expects(:uuid).returns("nonexisting")
      expect(provider.iscsi_target_name).to be_nil
    end
  end

  describe "#iscsi_target_ips" do
    it "should fetch the correct ip" do
      provider.facts = SpecHelper.json_fixture("equallogic-ess-rowc-rack7-eql6110-3_facts.json")
      expect(provider.iscsi_target_ips).to eq(["172.31.39.100"])
    end

    it "should return the correct failure value" do
      provider.facts = {}
      expect(provider.iscsi_target_ips).to eq([])
    end
  end

  describe "#volume_properties" do
    it "should find volumes if they exist" do
      provider.facts = SpecHelper.json_fixture("equallogic-ess-rowc-rack7-eql6110-3_facts.json")
      expect(provider.volume_properties("Newasm")).to eq(provider.facts["VolumesProperties"]["Newasm"])
      expect(provider.volume_properties("Newasm")).to_not be_nil
    end

    it "should return the correct failure value" do
      provider.facts = {}
      expect(provider.volume_properties("newasm")).to be_nil
    end
  end

  describe "#fc?" do
    it "should be false" do
      expect(provider.fc?).to be(false)
    end
  end

  describe "#initialize" do
    it "should specify the device run type" do
      expect(provider.puppet_type).to eq("asm::volume::equallogic")
    end
  end

  describe "#remove_server_from_volume!" do
    it "should use the deployment to process the volume" do
      deployment.expects(:process_storage).with(volume_component.to_hash)
      provider.remove_server_from_volume!(stub)
    end
  end

  describe "#esx_datastore_hash" do
    it "should create a valid datastore config" do
      type.related_server.stubs(:lookup_hostname).returns("172.28.10.63")
      expect(provider.esx_datastore(type.related_server, type.related_cluster, "absent")).to eq(expected_esx_datastore)
    end

    describe "when leaving a cluster" do
      it "should delegate to the cluster" do
        cluster = type.related_cluster
        cluster.stubs(:teardown?).returns(false)
        type.stubs(:related_cluster).returns(cluster)

        expected_hash = {}.merge(expected_esx_datastore).merge(cluster.provider.transport_config)

        servers = type.related_servers
        cluster.stubs(:related_servers).returns(servers)

        servers.each do |server|
          server.stubs(:teardown?).returns(false)
          server.stubs(:lookup_hostname).returns(server.hostip)
        end

        cluster.expects(:process_generic).with("vcenter-env10-vcenter.aidev.com", expected_hash, "apply", true, nil, "ff8080814dbf2d1d014dc2a280fd011f")
        type.leave_cluster!
      end
    end
  end

  describe "#datastore_require" do
    it "should return the require array" do
      ASM::PrivateUtil.stubs(:find_equallogic_iscsi_ip).returns("172.16.1.10")
      type.related_server.stubs(:lookup_hostname).returns("172.28.10.63")
      expect(provider.datastore_require(type.related_server)).to eq(["Asm::Datastore[172.28.10.63:M830-01:datastore_172.16.1.10]"])
    end
  end
end
