require 'spec_helper'
require 'asm/secrets'
require 'hashie'

describe ASM::Secrets do
  describe "#create" do

    it 'should require a secrets_source' do
      expect do
        ASM::Secrets.create(Hashie::Mash.new({}))
      end.to raise_error
    end

    it 'should require valid secrets_source' do
      expect do
        ASM::Secrets.create(Hashie::Mash.new({:secrets_source => 'unknown'}))
      end.to raise_error
    end

    it 'should require asm_api_user for local secrets' do
      expect do
        ASM::Secrets.create(Hashie::Mash.new({:secrets_source => :local,
                                              :asm_api_domain => 'domain'}))
      end.to raise_error
    end

    it 'should require asm_api_domain for local secrets' do
      expect do
        ASM::Secrets.create(Hashie::Mash.new({:secrets_source => :local,
                                              :asm_api_user => 'user'}))
      end.to raise_error
    end

    it 'should create local secrets' do
      secret = ASM::Secrets.create(Hashie::Mash.new({:secrets_source => :local,
                                                     :asm_api_user => 'user',
                                                     :asm_api_domain => 'domain'}))
      expect(secret.class).to eq(ASM::Secrets::LocalSecrets)
    end

    it 'should require url.asm_secrets for rest secrets' do
      expect do
        ASM::Secrets.create(Hashie::Mash.new({:secrets_source => 'rest'}))
      end.to raise_error
    end

    it 'should create rest secrets' do
      conf = Hashie::Mash.new
      conf.secrets_source = :rest
      conf.url!.asm_secrets = 'http://dellasm:8081/asm/secret'
      secret = ASM::Secrets.create(conf)
      expect(secret.class).to eq(ASM::Secrets::RestSecrets)
    end
  end

  describe("LocalSecrets#decrypt_token") do
    let(:request_token_1) {stub(:params => { "token_key" => "ASMTOKEN-891fb5a3-91aa-291c-4044-9a53234bc422"})}
    let(:request_token_2) {stub(:params => { "token_key" => "ASMTOKEN-554015df-ee46-59a2-8938-328140a6856d"})}
    let(:request_token_bad) {stub(:params => { "token_key" => "ASMTOKEN-DOES-NOT-EXIST"})}
    let(:cert_name) {"agent-mikewin193"}
    let(:local_secrets) {ASM::Secrets.create(Hashie::Mash.new({:secrets_source => :local,
                                                     :asm_api_user => 'user',
                                                     :asm_api_domain => 'domain'}))}
    let(:serverdata_one_token) {{"ASMTOKEN-891fb5a3-91aa-291c-4044-9a53234bc422" => {"timestamp" => "1476030112", "data" => "ASMCRED-dummy-encrypted-password-string-2"}}}
    let(:serverdata_multiple_tokens) {{"ASMTOKEN-891fb5a3-91aa-291c-4044-9a53234bc422" => {"timestamp" => "1476030112", "data" => "ASMCRED-dummy-encrypted-password-string-2"},
                                      "ASMTOKEN-554015df-ee46-59a2-8938-328140a6856d" => {"timestamp" => "1476030223", "data" => "ASMCRED-dummy-encrypted-password-string"}}}
    let(:serverdata_one_valid_token) {{"ASMTOKEN-891fb5a3-91aa-291c-4044-9a53234bc422" => {"timestamp" => Time.now.to_i, "data" => "ASMCRED-dummy-encrypted-password-string-2"}}}

    it("Should raise an exception if serverdata file does not exist") do
      ASM::PrivateUtil.expects(:get_serverdata).with(cert_name).returns nil
      expect{local_secrets.decrypt_token(cert_name, request_token_1)}.to raise_error("Token data is not available for #{cert_name}")
    end

    it("Should raise an exception if specified token does not exist") do
      ASM::PrivateUtil.expects(:get_serverdata).with(cert_name).returns(serverdata_one_token.to_json)
      expect{local_secrets.decrypt_token(cert_name, request_token_bad)}.to raise_error("Token data is not available for certname #{cert_name}, token key ASMTOKEN-DOES-NOT-EXIST")
    end

    it("Should delete serverdata file if only existing token matches and is expired") do
      ASM::PrivateUtil.expects(:get_serverdata).with(cert_name).returns(serverdata_one_token.to_json)
      ASM::PrivateUtil.expects(:delete_serverdata).with(cert_name).returns(nil)
      expect{local_secrets.decrypt_token(cert_name, request_token_1)}.to raise_error("Token data is invalid or has expired for certname #{cert_name}, token key ASMTOKEN-891fb5a3-91aa-291c-4044-9a53234bc422")
    end

    it("Should rewrite serverdata file matching token is expired") do
      ASM::PrivateUtil.expects(:get_serverdata).with(cert_name).returns(serverdata_multiple_tokens.to_json)
      ASM::PrivateUtil.expects(:write_serverdata).with(cert_name, serverdata_one_token).returns(nil)
      expect{local_secrets.decrypt_token(cert_name, request_token_2)}.to raise_error("Token data is invalid or has expired for certname #{cert_name}, token key ASMTOKEN-554015df-ee46-59a2-8938-328140a6856d")
    end

    it("Should return decrypted password for valid token") do
      ASM::PrivateUtil.expects(:get_serverdata).with(cert_name).returns(serverdata_one_valid_token.to_json)
      ASM::CipherLocal.expects(:decrypt_string).with("dummy-encrypted-password-string-2").returns("DECRYPTED_PASSWORD_STRING")
      expect(local_secrets.decrypt_token(cert_name, request_token_1)).to eq("DECRYPTED_PASSWORD_STRING")
    end
  end
end
